# -*- coding: utf-8 -*-
# Copyright 2015-2019 grafana-dashboard-builder contributors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from __future__ import unicode_literals

from grafana_dashboards.common import get_component_type
from grafana_dashboards.components.axes import Yaxes
from grafana_dashboards.components.base import JsonListGenerator, JsonGenerator
from grafana_dashboards.components.links import Links
from grafana_dashboards.components.targets import Targets

__author__ = 'Jakub Plichta <jakub.plichta@gmail.com>'


class Panels(JsonListGenerator):
    def __init__(self, data, registry):
        super(Panels, self).__init__(data, registry, PanelsItemBase)


class PanelsItemBase(JsonGenerator):
    pass


class Graph(PanelsItemBase):

    _copy_fields = {'stack', 'fill', 'aliasColors', 'leftYAxisLabel', 'bars', 'lines', 'linewidth', 'y_formats',
                    'x-axis', 'y-axis', 'points', 'pointradius', 'percentage', 'steppedLine', 'repeat',
                    'repeatDirection', 'decimals', 'minSpan', 'datasource', 'description'}

    def gen_json_from_data(self, data, context):
        panel_json = super(Graph, self).gen_json_from_data(data, context)
        panel_json.update({
            'type': 'graph',
            'title': self.data.get('title', None),
            'span': self.data.get('span', 12),
        })
        targets = self.data.get('targets', [])
        if 'target' in self.data:
            targets.append(self.data['target'])
        self._create_component(panel_json, Targets, {'targets': targets})
        panel_json['nullPointMode'] = self.data.get('nullPointMode', 'null')
        grid_data = self.data.get('grid', {}) or {}
        if 'grid' in self.data or 'y_formats' in self.data:
            panel_json['grid'] = {
                'leftMax': grid_data.get('leftMax', None),
                'rightMax': grid_data.get('rightMax', None),
                'leftMin': grid_data.get('leftMin', None),
                'rightMin': grid_data.get('rightMin', None),
                'threshold1': grid_data.get('threshold1', None),
                'threshold2': grid_data.get('threshold2', None),
                'threshold1Color': grid_data.get('threshold1Color', 'rgba(216, 200, 27, 0.27)'),
                'threshold2Color': grid_data.get('threshold2Color', 'rgba(234, 112, 112, 0.22)')
            }
        if 'legend' in self.data:
            panel_json['legend'] = {
                'show': self.data['legend'].get('show', True),
                'values': self.data['legend'].get('values', False),
                'min': self.data['legend'].get('min', False),
                'max': self.data['legend'].get('max', False),
                'current': self.data['legend'].get('current', False),
                'total': self.data['legend'].get('total', False),
                'avg': self.data['legend'].get('avg', False),
                'alignAsTable': self.data['legend'].get('alignAsTable', False),
                'rightSide': self.data['legend'].get('rightSide', False),
                'hideEmpty': self.data['legend'].get('hideEmpty', False),
                'hideZero': self.data['legend'].get('hideZero', False),
                'sideWidth': self.data['legend'].get('sideWidth', None)
            }
        if 'tooltip' in self.data:
            panel_json['tooltip'] = {
                'value_type': self.data['tooltip'].get('value_type', 'individual'),
                'shared': self.data['tooltip'].get('shared', False),
            }
        if 'seriesOverrides' in self.data:
            overrides = []
            for override in self.data['seriesOverrides']:
                for alias, settings in override.items():
                    to_add = {'alias': alias}
                    to_add.update(settings)
                    overrides.append(to_add)
            panel_json['seriesOverrides'] = overrides
        self._create_component(panel_json, Links, self.data)
        if (('leftYAxisLabel' in self.data
            or 'grid' in self.data and ('leftMin' in grid_data or 'leftMax' in grid_data))
                and ('y_formats' not in self.data)):
            panel_json['y_formats'] = ['short', 'short']
        panel_json['xaxis'] = self.data.get('xaxis', {'show': True, 'format': 'time'})
        self._create_component(panel_json, Yaxes, self.data)
        return panel_json

    def _create_component(self, panel_json, clazz, data):
        if get_component_type(clazz) in data:
            panel_json[get_component_type(clazz)] = self.registry.create_component(clazz, data).gen_json()


class SingleStat(PanelsItemBase):

    _copy_fields = {'prefix', 'postfix', 'nullText', 'format', 'thresholds', 'colorValue', 'colorBackground',
                    'colors', 'prefixFontSize', 'valueFontSize', 'postfixFontSize', 'maxDataPoints', 'datasource',
                    'repeat', 'repeatDirection', 'decimals', 'minSpan', 'description', 'colorPostfix'}

    def gen_json_from_data(self, data, context):
        panel_json = super(SingleStat, self).gen_json_from_data(data, context)
        panel_json.update({
            'type': 'singlestat',
            'title': data.get('title', None),
            'span': data.get('span', None),
            'nullPointMode': data.get('nullPointMode', 'null'),
            'valueName': data.get('valueName', 'current')
        })
        panel_json['targets'] = self.registry.create_component(Targets, data).gen_json() if 'targets' in data else []
        if 'sparkline' in data:
            panel_json['sparkline'] = {
                'show': True,
                'full': data['sparkline'].get('full', False),
                'lineColor': data['sparkline'].get('lineColor', 'rgb(31, 120, 193)'),
                'fillColor': data['sparkline'].get('fillColor', 'rgba(31, 118, 189, 0.18)')
            }
        if 'gauge' in data:
            panel_json['gauge'] = {
                'show': True,
                'minValue': data['gauge'].get('minValue', 0),
                'maxValue': data['gauge'].get('maxValue', 100),
                'thresholdMarkers': data['gauge'].get('thresholdMarkers', True),
                'thresholdLabels': data['gauge'].get('thresholdLabels', False)
            }
        if 'colors' not in data:
            panel_json['colors'] = [
                'rgba(50, 172, 45, 0.97)',
                'rgba(237, 129, 40, 0.89)',
                'rgba(245, 54, 54, 0.9)'
            ]
        if 'valueMaps' in data:
            panel_json['valueMaps'] = [{'value': value, 'op': '=', 'text': text} for value, text in
                                       data['valueMaps'].items()]
        if get_component_type(Links) in data:
            panel_json['links'] = self.registry.create_component(Links, data).gen_json()
        return panel_json


class Table(PanelsItemBase):
    # noinspection PySetFunctionToLiteral
    _copy_fields = {'fontSize', 'pageSize', 'showHeader', 'scroll', 'datasource', 'description'}

    def gen_json_from_data(self, data, context):
        panel_json = super(Table, self).gen_json_from_data(data, context)
        panel_json.update({
            'type': 'table',
            'title': data.get('title', None),
            'span': data.get('span', None),
            'targets': [{'target': v} for v in data.get('targets', [])],
            'transform': data.get('transform', None),
            'columns': [{'text': v, 'value': str(v).lower()} for v in data.get('columns', [])]
        })
        panel_json['targets'] = self.registry.create_component(Targets, data).gen_json() if 'targets' in data else []

        if 'styles' in self.data:
            styles = []
            for override in self.data['styles']:
                for pattern, settings in override.items():
                    to_add = {'pattern': pattern}
                    to_add.update(settings)
                    styles.append(to_add)
            panel_json['styles'] = styles

        return panel_json


class Text(PanelsItemBase):
    _copy_fields = {'description'}

    def gen_json_from_data(self, data, context):
        panel_json = super(Text, self).gen_json_from_data(data, context)
        panel_json.update({
            'type': 'text',
            'title': data.get('title', None),
            'span': data.get('span', None),
            'mode': data.get('mode', 'text'),
            'content': data.get('content', '')
        })
        return panel_json


class Dashlist(PanelsItemBase):
    _copy_fields = {'headings', 'limit', 'recent', 'tags', 'query', 'description'}

    def gen_json_from_data(self, data, context):
        panel_json = super(Dashlist, self).gen_json_from_data(data, context)
        panel_json.update({
            'type': 'dashlist',
            'title': data.get('title', None),
            'span': data.get('span', 12),
            'search': 'query' in data or 'tags' in data,
            'starred': data.get('starred') or ('query' not in data and 'tags' not in data)
        })
        return panel_json


class Gauge(PanelsItemBase):
    _copy_fields = {'datasource', 'pluginVersion'}

    def gen_json_from_data(self, data, context):
        panel_json = super(Gauge, self).gen_json_from_data(data, context)
        panel_json.update({
            'type': 'gauge',
            'title': data.get('title', None),
            'span': data.get('span', 12),
            'timeFrom': data.get('timeFrom', None),
            'timeShift': data.get('timeShift', None)
        })
        panel_json['targets'] = self.registry.create_component(Targets, data).gen_json() if 'targets' in data else []
        options_data = self.data.get('options', {}) or {}
        field_options_data = options_data.get('fieldOptions', {}) or {}
        field_options = {
            'values': field_options_data.get('values', False),
            'calcs': field_options_data.get('calcs', ['mean']),
            'defaults': field_options_data.get('defaults',
                                               {'mappings': [], 'thresholds': {'mode': 'absolute', 'steps': []}}),
            'overrides': field_options_data.get('overrides', [])
        }
        panel_json['options'] = {
            'showThresholdMarkers': options_data.get('showThresholdMarkers', False),
            'showThresholdLabels': options_data.get('showThresholdLabels', False),
            'orientation': options_data.get('orientation', 'auto'),
            'fieldOptions': field_options
        }
        return panel_json


class Stat(PanelsItemBase):
    _copy_fields = {'datasource', 'pluginVersion'}

    def gen_json_from_data(self, data, context):
        panel_json = super(Stat, self).gen_json_from_data(data, context)
        panel_json.update({
            'type': 'stat',
            'title': data.get('title', None),
            'span': data.get('span', 12),
            'timeFrom': data.get('timeFrom', None),
            'timeShift': data.get('timeShift', None)
        })
        panel_json['targets'] = self.registry.create_component(Targets, data).gen_json() if 'targets' in data else []
        options_data = self.data.get('options', {}) or {}
        field_options_data = options_data.get('fieldOptions', {}) or {}
        field_options = {
            'values': field_options_data.get('values', False),
            'calcs': field_options_data.get('calcs', ['mean']),
            'defaults': field_options_data.get('defaults',
                                               {'mappings': [], 'thresholds': {'mode': 'absolute', 'steps': []}}),
            'overrides': field_options_data.get('overrides', [])
        }
        panel_json['options'] = {
            'graphMode': options_data.get('graphMode', 'area'),
            'colorMode': options_data.get('colorMode', 'value'),
            'justifyMode': options_data.get('justifyMode', 'auto'),
            'orientation': options_data.get('orientation', 'auto'),
            'fieldOptions': field_options
        }
        return panel_json


class BarGauge(PanelsItemBase):
    _copy_fields = {'datasource'}

    def gen_json_from_data(self, data, context):
        panel_json = super(BarGauge, self).gen_json_from_data(data, context)

        panel_json.update({
            'type': 'bargauge',
            'title': data.get('title', None),
            'span': data.get('span', 12),
            'timeFrom': data.get('timeFrom', None),
            'timeShift': data.get('timeShift', None)
        })
        panel_json['targets'] = self.registry.create_component(Targets, data).gen_json() if 'targets' in data else []
        options_data = self.data.get('options', {}) or {}
        field_options_data = options_data.get('fieldOptions', {}) or {}
        field_options = {
            'values': field_options_data.get('values', False),
            'calcs': field_options_data.get('calcs', ['last']),
            'defaults': field_options_data.get('defaults', None),
            'mappings': field_options_data.get('mappings', []),
            'thresholds': field_options_data.get('thresholds', []),
            'override': field_options_data.get('override', None)
        }
        panel_json['options'] = {
            'orientation': options_data.get('orientation', 'auto'),
            'displayMode': options_data.get('displayMode', 'lcd'),
            'fieldOptions': field_options
        }
        return panel_json



# -*- coding: utf-8 -*-
# Copyright 2015-2019 grafana-dashboard-builder contributors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from __future__ import unicode_literals

from grafana_dashboards.components.base import JsonListGenerator, JsonGenerator

__author__ = 'Jakub Plichta <jakub.plichta@gmail.com>'


class Annotations(JsonListGenerator):
    def __init__(self, data, registry):
        super(Annotations, self).__init__(data, registry, AnnotationsItemBase)


class AnnotationsItemBase(JsonGenerator):
    pass


class Annotation(AnnotationsItemBase):
    _copy_fields = {'datasource'}

    def gen_json_from_data(self, data, context):
        template_json = super(Annotation, self).gen_json_from_data(data, context)
        template_json['name'] = data['name']
        template_json['expr'] = data['expr']
        template_json['enable'] = data.get('enable', True)
        template_json['hide'] = data.get('hide', False)
        template_json['iconColor'] = data.get('iconColor', 'rgba(255, 96, 96, 1)')
        template_json['showIn'] = data.get('showIn', 0)
        return template_json



# -*- coding: utf-8 -*-
#  Copyright 2021 grafana-dashboard-builder contributors
#
#     Licensed under the Apache License, Version 2.0 (the "License");
#     you may not use this file except in compliance with the License.
#     You may obtain a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#     Unless required by applicable law or agreed to in writing, software
#     distributed under the License is distributed on an "AS IS" BASIS,
#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#     See the License for the specific language governing permissions and
#     limitations under the License.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from __future__ import unicode_literals

from grafana_dashboards.components.base import JsonListGenerator, JsonGenerator

__author__ = 'Jakub Plichta <jakub.plichta@gmail.com>'


class Templates(JsonListGenerator):
    def __init__(self, data, registry):
        super(Templates, self).__init__(data, registry, TemplatesItemBase)


class TemplatesItemBase(JsonGenerator):
    pass


class Query(TemplatesItemBase):

    def gen_json_from_data(self, data, context):
        super(Query, self).gen_json_from_data(data, context)
        processed_parts = []
        queries = []
        if not data.get('query'):
            return queries
        if 'name' in data:
            template_json = {
                'type': 'query',
                'name': data['name'],
                'query': data['query']
            }
            if 'refresh' in data:
                template_json['refresh'] = data['refresh']
            elif 'options' in data:
                template_json['refresh'] = 0
            else:
                template_json['refresh'] = 1
            if 'datasource' in data:
                template_json['datasource'] = data['datasource']
            self._copy_data(template_json, data)
            queries.append(template_json)
        else:
            refresh_only_first = data.get('refresh-only-first', False)
            for query_part in data['query'].split('.'):
                if query_part.startswith('$'):
                    is_first = False if queries else True
                    query = query_part[1:]
                    metric = '*'
                    template_json = {
                        'type': 'query',
                        'refresh_on_load': not refresh_only_first or is_first,
                        'name': query,
                        'refresh': int(not refresh_only_first or is_first)
                    }
                    if 'datasource' in data:
                        template_json['datasource'] = data['datasource']
                    if query in data:
                        query_config = data[query]
                        metric = query_config.get('metric', metric)
                        self._copy_data(template_json, query_config)

                    template_json['query'] = '.'.join(processed_parts + [metric])
                    queries.append(template_json)
                processed_parts.append(query_part)
        return queries

    @staticmethod
    def _copy_data(target, source):
        if 'current' in source:
            current = source['current']
            target['current'] = {
                'text': current,
                'value': current
            }
        if 'options' in source:
            target['options'] = [{'text': option, 'value': option} for option in
                                 (source['options'])]
        for key in ['regex', 'multi', 'includeAll', 'hide', 'allFormat', 'allValue']:
            if key in source:
                target[key] = source[key]


class EnumeratedTemplateBase(TemplatesItemBase):

    def __init__(self, data, registry, template_type, refresh):
        super(EnumeratedTemplateBase, self).__init__(data, registry)
        self._template_type = template_type
        self._refresh = refresh

    def gen_json_from_data(self, data, context):
        template_json = super(EnumeratedTemplateBase, self).gen_json_from_data(data, context)
        template_json.update({
            'type': self._template_type,
            'refresh_on_load': False,
            'datasource': None,
            'name': data['name'],
            'query': ','.join([str(options) for options in data['options']]),
            'refresh': self._refresh
        })
        if 'current' in data:
            current = data['current']
            template_json['current'] = {
                'text': current,
                'value': current
            }
        if 'options' in data:
            template_json['options'] = [{'text': option, 'value': option} for option in
                                        (data['options'])]
        return template_json


class CustomTemplate(EnumeratedTemplateBase):
    def __init__(self, data, registry):
        super(CustomTemplate, self).__init__(data, registry, 'custom', 0)


class IntervalTemplate(EnumeratedTemplateBase):
    def __init__(self, data, registry):
        super(IntervalTemplate, self).__init__(data, registry, 'interval', 2)

    def gen_json_from_data(self, data, context):
        template_json = super(IntervalTemplate, self).gen_json_from_data(data, context)
        if 'auto' in data:
            template_json['auto'] = True
            template_json['auto_count'] = (data['auto'] or {}).get('count', 30)
            template_json['auto_min'] = (data['auto'] or {}).get('min', '10s')

            auto_option = {'text': 'auto', 'value': '$__auto_interval'}
            template_json.setdefault('options', []).append(auto_option)

            if data.get('current') == 'auto':
                template_json['current'] = auto_option
        return template_json


class DatasourceTemplate(TemplatesItemBase):
    def gen_json_from_data(self, data, context):
        template_json = super(DatasourceTemplate, self).gen_json_from_data(data, context)
        template_json.update({
            'type': 'datasource',
            'name': data.get('name', 'datasource'),
            'query': data['query']
        })
        if 'current' in data:
            current = data['current']
            template_json['current'] = {
                'text': current,
                'value': current
            }
        if 'regex' in data:
            template_json['regex'] = data['regex']

        return template_json



# -*- coding: utf-8 -*-
# Copyright 2015-2019 grafana-dashboard-builder contributors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from __future__ import unicode_literals

from grafana_dashboards.common import get_component_type
from grafana_dashboards.components.base import JsonListGenerator, JsonGenerator
from grafana_dashboards.errors import UnregisteredComponentError

try:
    basestring
except NameError:
    basestring = str

__author__ = 'Jakub Plichta <jakub.plichta@gmail.com>'


class Targets(JsonListGenerator):
    def __init__(self, data, registry):
        super(Targets, self).__init__(data, registry, TargetsItemBase)

    def gen_item_json(self, items, result_list):
        try:
            super(Targets, self).gen_item_json(items, result_list)
        except UnregisteredComponentError:
            result_list.append(
                self.registry.create_component(GraphiteTarget, {get_component_type(GraphiteTarget): items}).gen_json()
            )


class TargetsItemBase(JsonGenerator):
    pass


class GraphiteTarget(TargetsItemBase):

    def gen_json_from_data(self, data, context):
        template_json = super(GraphiteTarget, self).gen_json_from_data(data, context)
        if isinstance(data, basestring):
            template_json['target'] = data
        else:
            template_json['target'] = data['target']
        return template_json


class PrometheusTarget(TargetsItemBase):
    _copy_fields = {'format', 'hide', 'intervalFactor', 'legendFormat', 'step', 'instant', 'interval'}

    def gen_json_from_data(self, data, context):
        template_json = super(PrometheusTarget, self).gen_json_from_data(data, context)
        template_json['expr'] = data['expr']
        return template_json


class InfluxdbTarget(TargetsItemBase):
    _copy_fields = {'alias'}

    def gen_json_from_data(self, data, context):
        template_json = super(InfluxdbTarget, self).gen_json_from_data(data, context)
        template_json['query'] = data['query']
        template_json['dsType'] = 'influxdb'
        template_json['rawQuery'] = True
        return template_json


class ElasticTarget(TargetsItemBase):
    _copy_fields = {'bucketAggs', 'hide', 'metrics', 'refId', 'timeField'}

    def gen_json_from_data(self, data, context):
        template_json = super(ElasticTarget, self).gen_json_from_data(data, context)
        template_json['query'] = data['query']
        return template_json



# -*- coding: utf-8 -*-
# Copyright 2015-2019 grafana-dashboard-builder contributors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from __future__ import unicode_literals

import logging
import os

from grafana_dashboards.client.connection import (KerberosConnection,
                                                  BearerAuthConnection,
                                                  BasicAuthConnection,
                                                  SSLAuthConnection)
from grafana_dashboards.exporter import DashboardExporter

__author__ = 'Jakub Plichta <jakub.plichta@gmail.com>'


logger = logging.getLogger(__name__)


class GrafanaExporter(DashboardExporter):
    def __init__(self, **kwargs):
        super(GrafanaExporter, self).__init__()
        self._host = os.getenv('GRAFANA_HOST', kwargs.get('host'))
        password = os.getenv('GRAFANA_PASSWORD', kwargs.get('password'))
        username = os.getenv('GRAFANA_USERNAME', kwargs.get('username'))
        auth_token = os.getenv('GRAFANA_TOKEN', kwargs.get('token'))
        use_kerberos = os.getenv('GRAFANA_USE_KERBEROS', kwargs.get('use_kerberos'))
        client_crt = os.getenv('GRAFANA_SSL_CLIENT_CRT', kwargs.get('ssl_client_crt'))

        if use_kerberos:
            self._connection = KerberosConnection(self._host)
        elif auth_token:
            self._connection = BearerAuthConnection(auth_token, self._host)
        elif client_crt:
            client_key = os.getenv('GRAFANA_SSL_CLIENT_KEY', kwargs.get('ssl_client_key'))
            derived_key_path = os.path.splitext(client_crt)[0] + '.key'
            # pull the separate key also if not given explicitly and derived filename exists
            if client_key or (not client_key and os.path.exists(derived_key_path)):
                cert_bundle = (client_crt, client_key if client_key else derived_key_path)
            # otherwise assume bundled PEM
            else:
                cert_bundle = client_crt
            self._connection = SSLAuthConnection(self._host, cert_bundle)
        else:
            self._connection = BasicAuthConnection(username, password, self._host)

    def process_dashboard(self, project_name, dashboard_name, dashboard_data):
        super(GrafanaExporter, self).process_dashboard(project_name, dashboard_name, dashboard_data)
        body = {'overwrite': True, 'dashboard': dashboard_data}

        if 'folderId' in dashboard_data:
            body.update({'folderId': dashboard_data['folderId']})

        logger.info("Uploading dashboard '%s' to %s", dashboard_name, self._host)
        self._connection.make_request('/api/dashboards/db', body)



# -*- coding: utf-8 -*-
# Copyright 2015-2019 grafana-dashboard-builder contributors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from __future__ import unicode_literals

import base64
import json
import logging

try:
    from cookielib import CookieJar
except ImportError:
    from http.cookiejar import CookieJar
try:
    from urllib2 import build_opener, HTTPHandler, HTTPSHandler, HTTPCookieProcessor, HTTPDefaultErrorHandler, \
        Request, BaseHandler
except ImportError:
    from urllib.request import build_opener, HTTPHandler, HTTPSHandler, HTTPCookieProcessor, HTTPDefaultErrorHandler, \
        Request, BaseHandler
try:
    from urlparse import urlparse
except ImportError:
    from urllib.parse import urlparse

import requests
from requests_kerberos import HTTPKerberosAuth

__author__ = 'Jakub Plichta <jakub.plichta@gmail.com>'

logger = logging.getLogger(__name__)


class BaseConnection(object):
    _headers = {
        'Content-type': 'application/json',
        'Accept': 'application/json'
    }

    def __init__(self, host, auth_header, debug=0):
        self._host = host
        self._headers['Authorization'] = auth_header

        self._opener = build_opener(HTTPHandler(debuglevel=debug),
                                    HTTPSHandler(debuglevel=debug),
                                    HTTPCookieProcessor(CookieJar()),
                                    LoggingHandler(),
                                    HTTPDefaultErrorHandler())

    def make_request(self, uri, body=None):
        request = Request('{0}{1}'.format(self._host, uri),
                          json.dumps(body).encode('utf-8') if body else None,
                          headers=self._headers)
        response_body = self._opener.open(request).read()
        return {} if (response_body is None or response_body == '') else json.loads(response_body)


class BasicAuthConnection(BaseConnection):
    def __init__(self, username, password, host, debug=0):
        logger.debug('Creating new connection with username=%s host=%s', username, host)

        base64string = base64.encodebytes(('%s:%s' % (username, password)).encode('utf-8')).replace(b'\n', b'')

        super(BasicAuthConnection, self).__init__(host, b'Basic ' + base64string, debug)


class BearerAuthConnection(BaseConnection):
    def __init__(self, token, host, debug=0):
        logger.debug('Creating new connection with token=%s host=%s', token[:5], host)

        super(BearerAuthConnection, self).__init__(host, 'Bearer %s' % token.strip(), debug)


class LoggingHandler(BaseHandler):
    def __init__(self):
        pass

    # noinspection PyMethodMayBeStatic
    def http_request(self, request):
        path = urlparse(request.get_full_url()).path
        logger.debug('Sending request: method=%s uri=%s', request.get_method(), path)
        return request

    # noinspection PyMethodMayBeStatic,PyUnusedLocal
    def http_response(self, request, response):
        logger.debug('Response received: status=%s msg=%s', response.getcode(), response.msg)
        return response

    https_request = http_request
    https_response = http_response


class KerberosConnection(object):
    def __init__(self, host):
        logger.debug('Creating new kerberos connection with host=%s', host)
        self._host = host

    def make_request(self, uri, body=None):
        response = requests.post('{0}{1}'.format(self._host, uri), json=body, auth=HTTPKerberosAuth(), verify=False)
        return response.json()


class SSLAuthConnection(object):
    def __init__(self, host, cert_bundle, debug=0):
        logger.debug('Using SSL client cert from "%s" with host=%s', cert_bundle, host)
        self._host = host
        self._cert = cert_bundle

    def make_request(self, uri, body=None):
        response = requests.post('{0}{1}'.format(self._host, uri), json=body, cert=self._cert)
        return response.json()
